@@ -8,9 +8,6 @@
 #' @name RawDataGetters
 NULL
 
-
-## -------- getRawData --------
-
 #' @aliases getRawData
 #'
 #' @details `getRawData()` extracts the raw count table.
@@ -38,8 +35,6 @@ setMethod(
 )
 
 
-## -------- getNumCells --------
-
 #' @aliases getNumCells
 #'
 #' @details `getNumCells()` extracts the number of cells in the sample (\eqn{m})
@@ -64,8 +59,6 @@ setMethod(
 )
 
 
-## -------- getNumGenes --------
-
 #' @aliases getNumGenes
 #'
 #' @details `getNumGenes()` extracts the number of genes in the sample (\eqn{n})
@@ -90,8 +83,6 @@ setMethod(
 )
 
 
-## -------- getCells --------
-
 #' @aliases getCells
 #'
 #' @details `getCells()` extract all cells in the dataset.
@@ -116,8 +107,6 @@ setMethod(
 )
 
 
-## -------- getGenes --------
-
 #' @aliases getGenes
 #'
 #' @details `getGenes()` extract all genes in the dataset.
@@ -142,8 +131,6 @@ setMethod(
 )
 
 
-## -------- getZeroOneProj --------
-
 #' @aliases getZeroOneProj
 #'
 #' @details `getZeroOneProj()` extracts the raw count table where any
@@ -170,8 +157,6 @@ setMethod(
 )
 
 
-## -------- getCellsSize --------
-
 #' @aliases getCellsSize
 #'
 #' @details `getCellsSize()` extracts the cell raw library size.
@@ -198,8 +183,6 @@ setMethod(
 )
 
 
-## -------- getNumExpressedGenes --------
-
 #' @aliases getNumExpressedGenes
 #'
 #' @details `getNumExpressedGenes()` extracts the number of genes expressed for
@@ -228,8 +211,6 @@ setMethod(
 )
 
 
-## -------- getGenesSize --------
-
 #' @aliases getGenesSize
 #'
 #' @details `getGenesSize()` extracts the genes raw library size.
@@ -256,8 +237,6 @@ setMethod(
 )
 
 
-## -------- getNumOfExpressingCells --------
-
 #' @aliases getNumOfExpressingCells
 #'
 #' @details `getNumOfExpressingCells()` extracts, for each gene, the number of
@@ -303,8 +282,6 @@ setMethod(
 #' @name HandleMetaData
 NULL
 
-## -------- getMetadataDataset --------
-
 #' @aliases getMetadataDataset
 #'
 #' @details `getMetadataDataset()` extracts the meta-data stored for the
@@ -341,7 +318,6 @@ setMethod(
   }
 )
 
-## -------- getMetadataElement --------
 
 #' @aliases getMetadataElement
 #'
@@ -374,7 +350,6 @@ setMethod(
   }
 )
 
-## -------- getMetadataGenes --------
 
 #' @aliases getMetadataGenes
 #'
@@ -400,8 +375,6 @@ setMethod(
 )
 
 
-## -------- getMetadataCells --------
-
 #' @aliases getMetadataCells
 #'
 #' @details `getMetadataCells()` extracts the meta-data stored for the cells
@@ -426,8 +399,6 @@ setMethod(
 )
 
 
-## -------- getDims --------
-
 #' @aliases getDims
 #'
 #' @details `getDims()` extracts the sizes of all slots of the `COTAN` object
@@ -460,8 +431,6 @@ setMethod(
 
 # ------- `COTAN` estimated data accessors ------
 
-## -------- getNu --------
-
 #' @aliases getNu
 #'
 #' @details `getNu()` extracts the `nu` array (normalized cells' counts
@@ -494,8 +463,6 @@ setMethod(
 )
 
 
-## -------- getLambda --------
-
 #' @aliases getLambda
 #'
 #' @details `getLambda()` extracts the `lambda` array (mean expression for each
@@ -528,8 +495,6 @@ setMethod(
 )
 
 
-## -------- getDispersion --------
-
 #' @aliases getDispersion
 #'
 #' @details `getDispersion()` extracts the `dispersion` array (one value for
@@ -562,8 +527,6 @@ setMethod(
 )
 
 
-## -------- estimatorsAreReady --------
-
 #' @details `estimatorsAreReady()` checks whether the estimators arrays
 #'   `lambda`, `nu`, `dispersion` are available
 #'
@@ -592,8 +555,6 @@ estimatorsAreReady <- function(objCOTAN) {
 }
 
 
-## -------- getMu --------
-
 #' @aliases getMu
 #' @aliases calculateMu
 #'
@@ -623,8 +584,6 @@ getMu <- function(objCOTAN) {
 }
 
 
-## -------- getNuNormData --------
-
 #' @details `getNuNormData()` extracts the \eqn{\nu}*-normalized* count table
 #'   (i.e. where each column is divided by `nu`) and returns it
 #'
@@ -650,9 +609,6 @@ getNuNormData <- function(objCOTAN) {
   return(t(t(getRawData(objCOTAN)) * (1.0 / nu)))
 }
 
-
-## -------- getLogNormData --------
-
 #' @details `getLogNormData()` extracts the *log-normalized* count table (i.e.
 #'   where each column is divided by the [getCellsSize()]), takes its `log10`
 #'   and returns it.
@@ -677,9 +633,6 @@ getLogNormData <- function(objCOTAN) {
   return(log1p(normData) / log(10.0))
 }
 
-
-## -------- getNormalizedData --------
-
 #' @details `getNormalizedData()` is deprecated: please use [getNuNormData()] or
 #'   [getLogNormData()] directly as appropriate
 #'
@@ -704,8 +657,6 @@ getNormalizedData <- function(objCOTAN, retLog = FALSE) {
 }
 
 
-## -------- getProbabilityOfZero --------
-
 #' @details `getProbabilityOfZero()` gives for each cell and each gene the
 #'   probability of observing zero reads
 #'
@@ -886,8 +837,6 @@ setMethod(
 #' @name CalculatingCOEX
 NULL
 
-## -------- getGenesCoex --------
-
 #' @aliases getGenesCoex
 #'
 #' @details `getGenesCoex()` extracts a complete (or a partial after genes
@@ -970,8 +919,7 @@ NULL
 #' tables <- contingencyTables(objCOTAN, g1 = g1, g2 = g2)
 #' tables
 #'
-#' objCOTAN <- calculateCoex(objCOTAN, actOnCells = TRUE,
-#'                           optimizeForSpeed = FALSE)
+#' objCOTAN <- calculateCoex(objCOTAN, actOnCells = TRUE)
 #' stopifnot(isCoexAvailable(objCOTAN, actOnCells = TRUE, ignoreSync = TRUE))
 #' cellsCoex <- getCellsCoex(objCOTAN, zeroDiagonal = FALSE)
 #'
@@ -1030,8 +978,6 @@ setMethod(
 )
 
 
-## -------- getCellsCoex --------
-
 #' @aliases getCellsCoex
 #'
 #' @details `getCellsCoex()` extracts a complete (or a partial after cells
@@ -1097,8 +1043,6 @@ setMethod(
 )
 
 
-## -------- isCoexAvailable --------
-
 #' @aliases isCoexAvailable
 #'
 #' @details `isCoexAvailable()` allows to query whether the relevant `COEX`
@@ -1125,13 +1069,13 @@ setMethod(
   function(objCOTAN, actOnCells = FALSE, ignoreSync = FALSE) {
     if (isTRUE(actOnCells)) {
       isInSync <-
-        as.logical(getMetadataElement(objCOTAN, datasetTags()[["csync"]])[[1L]])
+        as.logical(getMetadataElement(objCOTAN, datasetTags()[["csync"]]))
 
       return(!is_empty(objCOTAN@cellsCoex) &&
                (isTRUE(ignoreSync) || isTRUE(isInSync)))
     } else {
       isInSync <-
-        as.logical(getMetadataElement(objCOTAN, datasetTags()[["gsync"]])[[1L]])
+        as.logical(getMetadataElement(objCOTAN, datasetTags()[["gsync"]]))
 
       return(!is_empty(objCOTAN@genesCoex) &&
                (isTRUE(ignoreSync) || isTRUE(isInSync)))
@@ -1140,8 +1084,6 @@ setMethod(
 )
 
 
-## -------- getGDI --------
-
 #' @aliases getGDI
 #'
 #' @details `getGDI()` extracts the genes' `GDI` array as it was stored by the
@@ -1175,8 +1117,6 @@ setMethod(
 )
 
 
-## -------- getClusterizations --------
-
 #' @aliases getClusterizations
 #'
 #' @details `getClusterizations()` extracts the list of the *clusterizations*
@@ -1293,9 +1233,6 @@ setMethod(
   }
 )
 
-
-## -------- getClusterizationName --------
-
 #' @aliases getClusterizationName
 #'
 #' @details `getClusterizationName()` normalizes the given *clusterization* name
@@ -1348,9 +1285,6 @@ setMethod(
   }
 )
 
-
-## -------- getClusterizationData --------
-
 #' @aliases getClusterizationData
 #'
 #' @details `getClusterizationData()` extracts the asked *clusterization* and
@@ -1396,9 +1330,7 @@ setMethod(
   }
 )
 
-
-## -------- getClusters --------
-
+#'
 #' @details `getClusters()` extracts the asked *clusterization* from the `COTAN`
 #'   object
 #'
@@ -1420,9 +1352,6 @@ getClusters <- function(objCOTAN, clName = "") {
   return(getClusterizationData(objCOTAN, clName = clName)[["clusters"]])
 }
 
-
-## -------- getClustersCoex --------
-
 #' @aliases getClustersCoex
 #'
 #' @details `getClustersCoex()` extracts the full `clusterCoex` member `list`
@@ -1462,9 +1391,6 @@ setMethod(
 #' @name HandlingConditions
 NULL
 
-
-## -------- getAllConditions --------
-
 #' @aliases getAllConditions
 #'
 #' @details `getAllConditions()` extracts the list of the *conditions* defined
@@ -1518,9 +1444,6 @@ setMethod(
   }
 )
 
-
-## -------- getConditionName --------
-
 #' @aliases getConditionName
 #'
 #' @details `getConditionName()` normalizes the given *condition* name or, if
@@ -1571,9 +1494,6 @@ setMethod(
   }
 )
 
-
-## -------- getCondition --------
-
 #' @aliases getCondition
 #'
 #' @details `getCondition()` extracts the asked *condition* from the `COTAN`
@@ -1615,8 +1535,6 @@ setMethod(
 )
 
 
-## -------- normalizeNameAndLabels --------
-
 #' @details `normalizeNameAndLabels()` takes a pair of name/labels and
 #'   normalize them based on the available information in the `COTAN` object
 #'
