@@ -19,45 +19,14 @@ datasetTags <- function() {
          , "sc.m"  = "scRNAseq method:"                                 # 2
          , "cond"  = "condition sample:"                                # 3
          , "cells" = "starting n. of cells:"                            # 4
-         , "gsync" = "genes' coex is in sync:"                          # 5
-         , "csync" = "cells' coex is in sync:"                          # 6
+         , "gsync" = "genes' COEX is in sync:"                          # 5
+         , "csync" = "cells' COEX is in sync:"                          # 6
          , "gbad"  = "genes' pairs fraction with small expected count:" # 7
          , "cbad"  = "cells' pairs fraction with small expected count:" # 8
          ))
 }
 
 
-#' @details `updateMetaInfo()` is an internal function: updates an information
-#'   `data.frame`
-#'
-#' @param meta the information `data.frame` to update
-#' @param tag the tag of the line
-#' @param value the value or the values to associate to the tag
-#'
-#' @returns `updateMetaInfo()` returns the updated `data.frame`
-#'
-#' @importFrom rlang is_empty
-#'
-#' @noRd
-#'
-updateMetaInfo <- function(meta, tag, value) {
-  # all values are converted to strings
-  newLine <- c(tag, paste0(value))
-
-  if (!is_empty(meta) && (tag %in% meta[[1L]])) {
-    # existing tag: update the value
-    rowPos <- which(meta[[1L]] %in% tag)
-  } else {
-    # new tag: add a new entry
-    rowPos <- nrow(meta) + 1L
-  }
-
-  meta[rowPos, seq_along(newLine)] <- newLine
-
-  return(meta)
-}
-
-
 #' @aliases initializeMetaDataset
 #'
 #' @details `initializeMetaDataset()` initializes meta-data data-set
@@ -187,7 +156,131 @@ setMethod(
     feCells <- getNumExpressedGenes(objCOTAN) >= threshold
 
     objCOTAN@metaCells <-
-      setColumnInDF(objCOTAN@metaCells, feCells, "feCells", getCells(objCOTAN))
+      setColumnInDF(objCOTAN@metaCells, feCells,
+                    "feCells", getCells(objCOTAN))
+
+    return(objCOTAN)
+  }
+)
+
+#' @aliases setLambda
+#'
+#' @details `setLambda()` adds a column to the genes' metadata with the lambda
+#'   (genes' counts averages) for the given batch
+#'
+#' @param objCOTAN a `COTAN` object
+#' @param lambda a named array that gives the values for lambda
+#'
+#' @returns `setLambda()` returns the updated `COTAN` object
+#'
+#' @importFrom assertthat assert_that
+#'
+#' @export
+#'
+#' @rdname RawDataCleaning
+#'
+setMethod(
+  "setLambda",
+  "COTAN",
+  function(objCOTAN, lambda) {
+    assert_that(length(lambda) == getNumGenes(objCOTAN))
+
+    if (TRUE) {
+      oldLambda <- getMetadataGenes(objCOTAN)[["lambda"]]
+      if (!identical(lambda, oldLambda)) {
+        # flag the COEX slots are out of sync (if any)!
+        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
+                                               datasetTags()[["gsync"]], FALSE)
+        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
+                                               datasetTags()[["csync"]], FALSE)
+      }
+    }
+
+    objCOTAN@metaGenes <-
+      setColumnInDF(objCOTAN@metaGenes, lambda,
+                    "lambda", getGenes(objCOTAN))
+
+    return(objCOTAN)
+  }
+)
+
+
+#' @aliases setDispersion
+#'
+#' @details `setDispersion()` adds a column to the genes' metadata with the
+#'   negative binomial dispersion factor for the given batch
+#'
+#' @param objCOTAN a `COTAN` object
+#' @param dispersion a named array that gives the values for the dispersion
+#'
+#' @returns `setDispersion()` returns the updated `COTAN` object
+#'
+#' @importFrom assertthat assert_that
+#'
+#' @export
+#'
+#' @rdname RawDataCleaning
+#'
+setMethod(
+  "setDispersion",
+  "COTAN",
+  function(objCOTAN, dispersion) {
+    assert_that(length(dispersion) == getNumGenes(objCOTAN))
+
+    if (TRUE) {
+      oldDispersion <-  getMetadataGenes(objCOTAN)[["dispersion"]]
+      if (!identical(dispersion, oldDispersion)) {
+        # flag the COEX slots are out of sync (if any)!
+        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
+                                               datasetTags()[["gsync"]], FALSE)
+        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
+                                               datasetTags()[["csync"]], FALSE)
+      }
+    }
+
+    objCOTAN@metaGenes <-
+      setColumnInDF(objCOTAN@metaGenes, dispersion,
+                    "dispersion", getGenes(objCOTAN))
+
+    return(objCOTAN)
+  }
+)
+
+#' @aliases setNu
+#'
+#' @details `setNu()`
+#'
+#' @param objCOTAN a `COTAN` object
+#' @param nu A named array that gives
+#'
+#' @returns `setNu()` returns the updated `COTAN` object
+#'
+#' @importFrom assertthat assert_that
+#'
+#' @export
+#'
+#' @rdname RawDataCleaning
+#'
+setMethod(
+  "setNu",
+  "COTAN",
+  function(objCOTAN, nu) {
+    assert_that(length(nu) == getNumCells(objCOTAN))
+
+    if (TRUE) {
+      oldNu <- getMetadataCells(objCOTAN)[["nu"]]
+      if (!identical(nu, oldNu)) {
+        # flag the COEX slots are out of sync (if any)!
+        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
+                                               datasetTags()[["gsync"]], FALSE)
+        objCOTAN@metaDataset <- updateMetaInfo(objCOTAN@metaDataset,
+                                               datasetTags()[["csync"]], FALSE)
+      }
+    }
+
+    objCOTAN@metaCells <-
+      setColumnInDF(objCOTAN@metaCells, nu,
+                    "nu", getCells(objCOTAN))
 
     return(objCOTAN)
   }
@@ -196,15 +289,15 @@ setMethod(
 
 #' @aliases storeGDI
 #'
-#' @details `storeGDI()` stored and already calculated genes' GDI `array` in a
+#' @details `storeGDI()` stored and already calculated genes' `GDI` `array` in a
 #'   `COTAN` object. It can be retrieved using the method [getGDI()]
 #'
 #' @param objCOTAN a `COTAN` object
-#' @param genesGDI the named genes' GDI `array` to store or the output
+#' @param genesGDI the named genes' `GDI` `array` to store or the output
 #'   `data.frame` of the function [calculateGDI()]
 #'
-#' @returns `storeGDI()` returns the given `COTAN` object  with updated
-#'   **GDI** genes' information
+#' @returns `storeGDI()` returns the given `COTAN` object  with updated `GDI`
+#'   genes' information
 #'
 #' @importFrom assertthat assert_that
 #'
@@ -270,13 +363,6 @@ setMethod(
                 msg = paste0("Asked to drop genes and/or cells",
                              " that were not present in the 'COTAN' object"))
 
-    if (length(genes) == 0L && length(cells) == 0L) {
-      logThis("Asked to drop no genes or cells", logLevel = 2L)
-    } else {
-      logThis(paste("Asked to drop", length(genes), "genes and",
-                    length(cells), "cells"), logLevel = 3L)
-    }
-
     genesPosToKeep <- which(!(getGenes(objCOTAN) %in% genes))
     cellsPosToKeep <- which(!(getCells(objCOTAN) %in% cells))
 
@@ -284,11 +370,23 @@ setMethod(
                 length(cellsPosToKeep) != 0L,
                 msg = "Asked to drop all genes and/or cells")
 
+    if (length(genesPosToKeep) == getNumGenes(objCOTAN) &&
+        length(cellsPosToKeep) == getNumCells(objCOTAN)) {
+      logThis("Asked to drop no genes or cells", logLevel = 2L)
+      # return original object
+      return(objCOTAN)
+    }
+
+    logThis(paste("Asked to drop", length(genes), "genes and",
+                  length(cells), "cells"), logLevel = 3L)
+
     # As all estimates would be wrong, a completely new object is created
     output <- COTAN(objCOTAN@raw[genesPosToKeep, cellsPosToKeep, drop = FALSE])
 
-    # Copy the meta data for the data-set
+    # Copy the meta data for the data-set thus erasing the sync bits
     output@metaDataset <- getMetadataDataset(objCOTAN)
+    output <- addElementToMetaDataset(output, datasetTags()[["gsync"]], FALSE)
+    output <- addElementToMetaDataset(output, datasetTags()[["csync"]], FALSE)
 
     # Filter the meta data for genes keeping those not related to estimates
     colsToKeep <- !(names(getMetadataGenes(objCOTAN)) %in%
@@ -388,7 +486,7 @@ setMethod(
 
       assert_that(identical(rownames(coexDF), getGenes(objCOTAN)),
                   setequal(colnames(coexDF), levels(clusters)),
-                  msg = "coex is not aligned to the given clusterization")
+                  msg = "COEX is not aligned to the given clusterization")
     }
 
     objCOTAN@metaCells <- setColumnInDF(objCOTAN@metaCells, clusters,
@@ -439,7 +537,7 @@ setMethod(
       assert_that(identical(rownames(coexDF), getGenes(objCOTAN)),
                   setequal(colnames(coexDF),
                            getClusters(objCOTAN, clName = internalName)),
-                  msg = "coex is not aligned to the given clusterization")
+                  msg = "COEX is not aligned to the given clusterization")
     }
 
     # this should not add any new elements to the list!
