@@ -4,11 +4,11 @@
 #' @title Calculating the `COEX` matrix
 #'
 #' @description These are the functions and methods used to calculate the
-#'   **COEX** matrices according to the `COTAN` model. From there it is possible
-#'   to calculate the associated *pValue* and the *GDI* (*Global Differential
+#'   `COEX` matrices according to the `COTAN` model. From there it is possible
+#'   to calculate the associated `p-value` and the `GDI` (*Global Differential
 #'   Expression*)
 #'
-#' @description The **COEX** matrix is defined by following formula:
+#' @description The `COEX` matrix is defined by following formula:
 #'
 #'   \deqn{\frac{\sum_{i,j \in \{\text{Y, N}\}}{
 #'                     (-1)^{\#\{i,j\}}\frac{O_{ij}-E_{ij}}{1 \vee E_{ij}}}}
@@ -16,8 +16,8 @@
 #'                             \frac{1}{1 \vee E_{ij}}}}}}
 #'
 #'   where \eqn{O} and \eqn{E} are the observed and expected contingency tables
-#'   and \eqn{n} is the relevant numerosity (the number of genes/cells depending
-#'   on given `actOnCells` flag).
+#'   and \eqn{n} is the relevant number of genes/cells (depending on given
+#'   `actOnCells` flag).
 #'
 #'   The formula can be more effectively implemented as:
 #'
@@ -58,24 +58,136 @@ NULL
 #'   the likelihood of the observed zero/one data
 #'
 #' @param objCOTAN a `COTAN` object
+#' @param formula a string indicating which function of the likelihood is
+#'   actually returned. Supported formulas are:
+#'   * `"raw"` just the likelihood (default):
+#'     \eqn{p^{(1-z)} \times (1-p)^z = (1.0 - z) p + z (1.0 - p)}
+#'   * `"log"` the log of the likelihood:
+#'     \eqn{(1.0 - z) \log(p) + z \log(1.0 - p)}
+#'   * `"der"` the derivative of the log of the likelihood:
+#'     \eqn{(1.0 - z) / p - z / (1.0 - p)}
+#'   * `"sLog"` the *signed* log of the likelihood:
+#'     \eqn{(1.0 - z) \log(p) - z \log(1.0 - p)}
 #'
-#' @returns `calculateLikelihoodOfObserved()` returns a `data.frame` with the
-#'   likelihood of the observed zero/one
+#'   where \eqn{z} is the *binarized projection* and \eqn{p} is the *probability
+#'   of zero*
+#'
+#' @returns `calculateLikelihoodOfObserved()` returns a `matrix` with the
+#'   selected *formula* of the likelihood of the observed zero/one
 #'
 #' @export
 #'
 #' @examples
-#' lh <- calculateLikelihoodOfObserved(objCOTAN)
+#' signedLikelhood <- calculateLikelihoodOfObserved(objCOTAN, formula = "sLog")
 #'
 #' @rdname CalculatingCOEX
 #'
-calculateLikelihoodOfObserved <- function(objCOTAN) {
-  zeroOne <- getZeroOneProj(objCOTAN)
+calculateLikelihoodOfObserved <- function(objCOTAN,
+                                          formula = "raw") {
+  rawData <- getRawData(objCOTAN)
 
-  probZero <- getProbabilityOfZero(objCOTAN)
+  # bound the probabilities to avoid exact zero or one
+  probZero <- pmin(pmax(getProbabilityOfZero(objCOTAN), 1.0e-8), 1.0 - 1.0e-8)
 
-  # estimate the likelihood of observed result
-  return((1.0 - zeroOne) * probZero + zeroOne * (1.0 - probZero))
+  res <-
+    switch(formula,
+           raw  = ifelse(rawData != 0L,       1.0-probZero,       probZero),
+           log  = ifelse(rawData != 0L,    log1p(-probZero),  log(probZero)),
+           der  = ifelse(rawData != 0L, -1.0/(1.0-probZero),  1.0/probZero),
+           sLog = ifelse(rawData != 0L,   -log1p(-probZero),  log(probZero)),
+           stop("Unknown formula: ", formula))
+
+  rm(probZero)
+
+  dim(res) <- dim(rawData)
+  dimnames(res) <- dimnames(rawData)
+
+  return(res)
+}
+
+#' @details `getDataMatrix()` gives for each cell and each gene the result of
+#'   the selected formula as function of the observed counts and their expected
+#'   value
+#'
+#' @param objCOTAN a `COTAN` object
+#' @param dataMethod selects the method to use to create the `data.frame` to
+#'   pass to the [UMAPPlot()]. To calculate, for each cell, a statistic for each
+#'   gene based on available data/model, the following methods are supported:
+#'   * `"RW", "Raw", "RawData"` uses the *raw* counts
+#'   * `"NN", "NuNorm", "Normalized"` uses the \eqn{\nu}*-normalized* counts
+#'   * `"LN", "LogNorm", "LogNormalized"` uses the *log-normalized* counts
+#'   (default)
+#'   * `"BI", "Bin", "Binarized"` uses the *binarized* data matrix
+#'   * `"BD", "BinDiscr", "BinarizedDiscrepancy"` uses the *difference* between
+#'   the *binarized* data matrix and the estimated *probability of one*
+#'   * `"AB", "AdjBin", "AdjBinarized"` uses the absolute value of
+#'   the *binarized discrepancy* above
+#'   * `"LH", "Like", "Likelihood"` uses the *likelihood* of *binarized*
+#'   data matrix
+#'   * `"LL", "LogLike", "LogLikelihood"` uses the *log-likelihood*
+#'   of *binarized* data matrix
+#'   * `"DL", "DerLogL", "DerivativeLogLikelihood"` uses the *derivative* of
+#'   the *log-likelihood* of *binarized* data matrix
+#'   * `"SL", "SignLogL", "SignedLogLikelihood"` uses the *signed log-likelihood*
+#'   of *binarized* data matrix
+#'
+#'   For the last four options see [calculateLikelihoodOfObserved()] for more
+#'   details
+#'
+#' @returns `getDataMatrix()` returns a `matrix` with the same shape as the
+#'   *raw* data
+#'
+#' @export
+#'
+#' @rdname CalculatingCOEX
+#'
+getDataMatrix <- function(objCOTAN,
+                          dataMethod = "") {
+  if (isEmptyName(dataMethod)) {
+    dataMethod <- "LogNormalized"
+  }
+
+  # A little helper to do non trivial binarized in less memory
+  binDiscr <- function(objCOTAN, formula) {
+    rawData <- getRawData(objCOTAN)
+    probZero <- getProbabilityOfZero(objCOTAN)
+
+    res <- switch(formula,
+                  raw  = ifelse(rawData != 0L, 1.0, 0.0),
+                  dis  = ifelse(rawData != 0L, probZero, probZero - 1.0),
+                  abs  = ifelse(rawData != 0L, probZero, 1.0 - probZero),
+                  stop("Unknown formula: ", formula))
+
+    rm(probZero)
+
+    dim(res) <- dim(rawData)
+    dimnames(res) <- dimnames(rawData)
+
+    return(res)
+  }
+
+  getLH <- function(objCOTAN, formula) {
+    return(calculateLikelihoodOfObserved(objCOTAN, formula))
+  }
+
+  dataMatrix <- as.matrix(switch(
+    dataMethod,
+    RW = , Raw      = , RawData                 = getRawData(objCOTAN),
+    NN = , NuNorm   = , Normalized              = getNuNormData(objCOTAN),
+    LN = , LogNorm  = , LogNormalized           = getLogNormData(objCOTAN),
+    BI = , Bin      = , Binarized               = getZeroOneProj(objCOTAN),
+    BD = , BinDiscr = , BinarizedDiscrepancy    = binDiscr(objCOTAN, "dis"),
+    AB = , AdjBin   = , AdjBinarized            = binDiscr(objCOTAN, "abs"),
+    LH = , Like     = , Likelihood              = getLH(objCOTAN, "raw"),
+    LL = , LogLike  = , LogLikelihood           = getLH(objCOTAN, "log"),
+    DL = , DerLogL  = , DerivativeLogLikelihood = getLH(objCOTAN, "der"),
+    SL = , SignLogL = , SignedLogLikelihood     = getLH(objCOTAN, "sLog"),
+    stop("Unrecognised `dataMethod` passed in: ", dataMethod)
+  ))
+
+  gc()
+
+  return(dataMatrix)
 }
 
 
@@ -817,9 +929,19 @@ contingencyTables <- function(objCOTAN, g1, g2) {
                        ncol = 2L, nrow = 2L, dimnames = dimnames)
 
   # estimated
+  lambda <- suppressWarnings(getLambda(objCOTAN))
+  assert_that(!is_empty(lambda),
+              msg = "`lambda` must not be empty, estimate it")
 
-  probZero <- funProbZero(getDispersion(objCOTAN)[c(g1, g2)],
-                          getLambda(objCOTAN)[c(g1, g2)] %o% getNu(objCOTAN))
+  nu <- suppressWarnings(getNu(objCOTAN))
+  assert_that(!is_empty(nu),
+              msg = "`nu` must not be empty, estimate it")
+
+  dispersion <- suppressWarnings(getDispersion(objCOTAN))
+  assert_that(!is_empty(dispersion),
+              msg = "`dispersion` must not be empty, estimate it")
+
+  probZero <- funProbZero(dispersion[c(g1, g2)], lambda[c(g1, g2)] %o% nu)
   rownames(probZero) <- c(g1, g2)
 
   if (anyNA(probZero)) {
@@ -848,7 +970,8 @@ contingencyTables <- function(objCOTAN, g1, g2) {
 
 # --------------- COEX and related matrices -----------
 
-# legacy code for cases when the torch library is not available
+## ------ Legacy implementation ------
+
 calculateCoex_Legacy <- function(objCOTAN, actOnCells, returnPPFract) {
   startTime <- Sys.time()
 
@@ -857,7 +980,8 @@ calculateCoex_Legacy <- function(objCOTAN, actOnCells, returnPPFract) {
   } else {
     kind <- "genes'"
   }
-  logThis(paste("Calculate", kind, "coex (legacy): START"), logLevel = 1L)
+
+  logThis(paste("Calculate", kind, "COEX (legacy): START"), logLevel = 1L)
 
   logThis(paste("Retrieving expected", kind, "contingency table"),
           logLevel = 3L)
@@ -874,7 +998,7 @@ calculateCoex_Legacy <- function(objCOTAN, actOnCells, returnPPFract) {
                 difftime(expectedTime, startTime, units = "secs")),
           logLevel = 3L)
 
-  logThis(paste("Calculating", kind, "coex normalization factor"),
+  logThis(paste("Calculating", kind, "COEX normalization factor"),
           logLevel = 3L)
 
   if (isTRUE(actOnCells)) {
@@ -922,8 +1046,8 @@ calculateCoex_Legacy <- function(objCOTAN, actOnCells, returnPPFract) {
                 difftime(observedTime, sqrtTime, units = "secs")),
           logLevel = 3L)
 
-  # coex estimation
-  logThis(paste("Estimating", kind, "coex"), logLevel = 3L)
+  # COEX estimation
+  logThis(paste("Estimating", kind, "COEX"), logLevel = 3L)
 
   coex <- coex * (observedYY@x - expectedYY@x)
 
@@ -936,7 +1060,7 @@ calculateCoex_Legacy <- function(objCOTAN, actOnCells, returnPPFract) {
   gc()
 
   endTime <- Sys.time()
-  logThis(paste("Calculate", kind, "coex elapsed time:",
+  logThis(paste("Calculate", kind, "COEX elapsed time:",
                 difftime(endTime, observedTime, units = "secs")),
           logLevel = 3L)
 
@@ -944,77 +1068,94 @@ calculateCoex_Legacy <- function(objCOTAN, actOnCells, returnPPFract) {
                 difftime(endTime, startTime, units = "secs")),
           logLevel = 2L)
 
-  logThis(paste("Calculate", kind, "coex (legacy): DONE"), logLevel = 1L)
+  logThis(paste("Calculate", kind, "COEX (legacy): DONE"), logLevel = 1L)
 
   return(list("coex" = coex, "ppf" = problematicPairsFraction))
 }
 
 
-# torch based code
-calculateCoex_Torch <- function(objCOTAN, returnPPFract, deviceStr) {
-  logThis(paste0("Calculate genes coex (torch) on device ", deviceStr,
-                 ": START"), logLevel = 1L)
+## ------ Torch implementation ------
 
+calculateCoex_Torch <- function(objCOTAN, returnPPFract, deviceStr) {
   startTime <- Sys.time()
 
-  gc()
-
-  useCuda <- startsWith(deviceStr, "cuda")
-
-  dtypeForCalc <- torch::torch_float32()
-
-  if (useCuda) {
-    torch::cuda_empty_cache()
-
-    # TODO: 16 bits are OK here?
-    halfDtypeForCalc <- torch::torch_float16()
-  } else {
-    halfDtypeForCalc <- torch::torch_float32()
-  }
-
-  device <- torch::torch_device(deviceStr)
-
-  probOne <- function(nu, lambda, a) {
-    zero     <- torch::torch_tensor( 0.0, device = device,
-                                    dtype = torch::torch_float64())
-    minusOne <- torch::torch_tensor(-1.0, device = device,
-                                    dtype = torch::torch_float64())
-
-    # Calculate terms based on conditions
-    term1 <- (a <= zero) *
-      torch::torch_exp(torch::torch_ger(nu, lambda) *
-                         (torch::torch_minimum(a, zero) + minusOne))
-
-    term2 <- (a >  zero) *
-      torch::torch_pow(one + torch::torch_ger(nu, lambda)
-                       * torch::torch_maximum(a, zero), minusOne / a)
-
-    invisible(term1$add_(term2)$add_(minusOne)$neg_())
-
-    return(term1)
-  }
+  logThis(paste0("Calculate genes COEX (torch) on device ", deviceStr,
+                 ": START"), logLevel = 1L)
 
   logThis("Retrieving expected genes contingency table", logLevel = 3L)
 
-  one <- torch::torch_tensor(1.0, device = device,
-                             dtype = dtypeForCalc)
-  m   <- torch::torch_tensor(getNumCells(objCOTAN),
-                             device = device, dtype = dtypeForCalc)
+  useCuda <- startsWith(deviceStr, "cuda")
 
-  expectedYY <- torch::torch_tensor(probOne(
-    torch::torch_tensor(getNu(objCOTAN),
-                        dtype = torch::torch_float64(), device = device),
-    torch::torch_tensor(getLambda(objCOTAN),
-                        dtype = torch::torch_float64(), device = device),
-    torch::torch_tensor(getDispersion(objCOTAN),
-                        dtype = torch::torch_float64(), device = device)),
-    device = device, dtype = dtypeForCalc)
+  dType <- torch::torch_float32()
 
-  if (useCuda) {
-    torch::cuda_empty_cache()
+  device <- torch::torch_device(deviceStr)
+
+  nCel <- getNumCells(objCOTAN)
+
+  probOne <- function(lambda, nu, disp) {
+    # Always process in cells‑blocks (rows) of size `numGenes`
+    numCells <- nu$size(1L)
+    numGenes <- lambda$size(1L)
+
+    # comparison masks
+    mif <- disp == -Inf
+    pif <- disp == +Inf
+    neg <- disp < 1.0e-5 & !mif # use neg expr for very small disp
+    pos <- !neg & !mif & !pif
+
+    # prepare output
+    out <- torch::torch_empty(c(numCells, numGenes),
+                              device = device, dtype = dType)
+
+    for (i in seq.int(1L, numCells, by = numGenes)) {
+      i_end  <- min(i + numGenes - 1L, numCells)
+
+      # outer product
+      mu_blk <- torch::torch_ger(nu[i:i_end], lambda)
+
+      out[i:i_end, mif] <- -Inf
+      out[i:i_end, neg] <- mu_blk[, neg] * (disp[neg] - 1.0)
+      out[i:i_end, pos] <- -torch::torch_reciprocal(disp[pos]) *
+        torch::torch_log1p(mu_blk[, pos] * disp[pos])
+      out[i:i_end, pif] <- 0.0
+
+      out[i:i_end, ] <- torch::torch_exp(out[i:i_end, ])
+
+      # copy to output
+      rm(mu_blk)
+    }
+
+    # finalize in place
+    invisible(out$sub_(1.0)$neg_())
+
+    return(out)
   }
 
-  expectedY <- torch::torch_sum(expectedYY, 1L, dtype = dtypeForCalc)
+  lambda <- suppressWarnings(getLambda(objCOTAN))
+  assert_that(!is_empty(lambda),
+              msg = "`lambda` must not be empty, estimate it")
+
+  nu <- suppressWarnings(getNu(objCOTAN))
+  assert_that(!is_empty(nu),
+              msg = "`nu` must not be empty, estimate it")
+
+  disp <- suppressWarnings(getDispersion(objCOTAN))
+  assert_that(!is_empty(disp),
+              msg = "`dispersion` must not be empty, estimate it")
+
+  # Build all three in dtype_calc:
+  lambda_t <- torch::torch_tensor(lambda, device = device, dtype = dType)
+  nu_t     <- torch::torch_tensor(nu,     device = device, dtype = dType)
+  disp_t   <- torch::torch_tensor(disp,   device = device, dtype = dType)
+
+  # Toss the originals to free CPU RAM:
+  rm(lambda, nu, disp)
+
+  expectedYY <- probOne(lambda_t, nu_t, disp_t)
+
+  rm(lambda_t, nu_t, disp_t)
+
+  expectedY <- torch::torch_sum(expectedYY, 1L, dtype = dType)
 
   expectedYY <- torch::torch_mm(torch::torch_t(expectedYY), expectedYY)
 
@@ -1023,59 +1164,65 @@ calculateCoex_Torch <- function(objCOTAN, returnPPFract, deviceStr) {
                 difftime(expectedTime, startTime, units = "secs")),
           logLevel = 3L)
 
-  logThis("Calculating genes coex normalization factor", logLevel = 3L)
 
-  coex <- torch::torch_maximum(expectedYY, one)$reciprocal_()
+  logThis("Calculating genes COEX normalization factor", logLevel = 3L)
 
   thresholdForPP <- 0.5
   problematicPairs <- NULL
+
+  # expectedYY
   if (isTRUE(returnPPFract)) {
     problematicPairs <-
       torch::torch_tensor(expectedYY < thresholdForPP,
                           dtype = torch::torch_bool(), device = device)
   }
+  # 1.0 / max(1.0, eYY)  [new tensor]
+  coex <- expectedYY$clamp_min(1.0)
+  invisible(coex$reciprocal_())
 
   # expectedYN
-  #TODO: use in-place torch_maximum_() as soon as it becomes available
   tmp <- expectedY$view(c(-1L, 1L)) - expectedYY
   if (isTRUE(returnPPFract)) {
-    invisible(problematicPairs$bitwise_or_(tmp < thresholdForPP))
+    mask <- tmp$lt(thresholdForPP)
+    invisible(problematicPairs$bitwise_or_(mask));
+    rm(mask)
   }
-  invisible(coex$add_(tmp$sub_(one)$relu_()$add_(one)$reciprocal_()))
+  # 1.0 / max(1.0, eYN) [in-place]
+  tmp <- tmp$clamp_min_(1.0)$reciprocal_()
+  invisible(coex$add_(tmp))
 
   rm(tmp)
-  if (useCuda) {
-    torch::cuda_empty_cache()
-  }
 
   # expectedNY
   tmp <- expectedY$view(c(1L, -1L)) - expectedYY
   if (isTRUE(returnPPFract)) {
-    invisible(problematicPairs$bitwise_or_(tmp < thresholdForPP))
+    mask <- tmp$lt(thresholdForPP)
+    invisible(problematicPairs$bitwise_or_(mask))
+    rm(mask)
   }
-  invisible(coex$add_(tmp$sub_(one)$relu_()$add_(one)$reciprocal_()))
+  # 1.0 / max(1.0, eNY) [in-place]
+  tmp <- tmp$clamp_min_(1.0)$reciprocal_()
+  invisible(coex$add_(tmp))
 
   rm(tmp)
-  if (useCuda) {
-    torch::cuda_empty_cache()
-  }
 
   # expectedNN
   tmp <- expectedYY - expectedY$view(c(-1L, 1L))
-  invisible(expectedY$sub_(m))
+  invisible(expectedY$sub_(nCel))
   invisible(tmp$sub_(expectedY$view(c(1L, -1L))))
   if (isTRUE(returnPPFract)) {
-    invisible(problematicPairs$bitwise_or_(tmp < thresholdForPP))
+    mask <- tmp$lt(thresholdForPP)
+    invisible(problematicPairs$bitwise_or_(mask))
+    rm(mask)
   }
-  invisible(coex$add_(tmp$sub_(one)$relu_()$add_(one)$reciprocal_()))
+  # 1.0 / max(1.0, eNN) [in-place]
+  tmp <- tmp$clamp_min_(1.0)$reciprocal_()
+  invisible(coex$add_(tmp))
 
   rm(tmp, expectedY)
   gc()
-  if (useCuda) {
-    torch::cuda_empty_cache()
-  }
 
-  invisible(coex$divide_(m)$sqrt_())
+  invisible(coex$div_(nCel)$sqrt_())
 
   # count problematic pairs
   problematicPairsFraction <- NA
@@ -1090,23 +1237,19 @@ calculateCoex_Torch <- function(objCOTAN, returnPPFract, deviceStr) {
                   problematicPairsFraction), logLevel = 3L)
 
     rm(problematicPairs)
-    if (useCuda) {
-      torch::cuda_empty_cache()
-    }
   }
 
   sqrtTime <- Sys.time()
-  logThis(paste("Calculating genes coex normalization factor elapsed time:",
+  logThis(paste("Calculating genes COEX normalization factor elapsed time:",
                 difftime(sqrtTime, expectedTime, units = "secs")),
           logLevel = 3L)
 
   logThis("Retrieving observed genes yes/yes contingency table", logLevel = 3L)
 
   # observedYY
-  observedYY <- torch::torch_tensor(
-    as.matrix(getRawData(objCOTAN)),
-    device = device, dtype = torch::torch_int16())
-  observedYY <- torch::torch_tensor(observedYY != 0L, dtype = halfDtypeForCalc)
+  observedYY <-
+    torch::torch_tensor(suppressWarnings(as.matrix(getZeroOneProj(objCOTAN))),
+                        device = device, dtype = dType)
 
   observedYY <- torch::torch_mm(observedYY, torch::torch_t(observedYY))
 
@@ -1115,13 +1258,11 @@ calculateCoex_Torch <- function(objCOTAN, returnPPFract, deviceStr) {
                 difftime(observedTime, sqrtTime, units = "secs")),
           logLevel = 3L)
 
-  invisible(expectedYY$subtract_(observedYY))
-  invisible(coex$multiply_(expectedYY$neg()))
+  invisible(observedYY$sub_(expectedYY))
+
+  invisible(coex$mul_(observedYY))
 
   rm(expectedYY, observedYY)
-  if (useCuda) {
-    torch::cuda_empty_cache()
-  }
 
   ret <- pack(forceSymmetric(as(as.matrix(coex$cpu()), "denseMatrix")))
   rownames(ret) <- colnames(ret) <- getGenes(objCOTAN)
@@ -1133,14 +1274,14 @@ calculateCoex_Torch <- function(objCOTAN, returnPPFract, deviceStr) {
   }
 
   endTime <- Sys.time()
-  logThis(paste("Calculate genes coex elapsed time:",
+  logThis(paste("Calculate genes COEX elapsed time:",
                 difftime(endTime, observedTime, units = "secs")),
           logLevel = 3L)
   logThis(paste("Total calculations elapsed time:",
                 difftime(endTime, startTime, units = "secs")),
           logLevel = 2L)
 
-  logThis(paste0("Calculate genes coex (torch) on device ", deviceStr,
+  logThis(paste0("Calculate genes COEX (torch) on device ", deviceStr,
                  ": DONE"), logLevel = 1L)
 
   return(list("coex" = ret, "ppf" = problematicPairsFraction))
@@ -1195,8 +1336,8 @@ setMethod(
 
     if (isTRUE(actOnCells)) {
       if (isTRUE(optimizeForSpeed)) {
-        warning("The 'torch' package is not supported yet for cells' COEX",
-                " Falling back to legacy code.")
+        warning("The 'torch' package is not supported yet for cells' COEX:",
+                " falling back to legacy code")
       }
       c(coex, problematicPairsFraction) %<-%
         calculateCoex_Legacy(objCOTAN, actOnCells = TRUE,
@@ -1268,12 +1409,14 @@ calculatePartialCoex <- function(objCOTAN,
                                  zeroOne = NULL,
                                  actOnCells = FALSE,
                                  optimizeForSpeed = TRUE) {
-    if (isTRUE(actOnCells)) {
+  startTime <- Sys.time()
+
+  if (isTRUE(actOnCells)) {
       kind <- "cells'"
     } else {
       kind <- "genes'"
     }
-    logThis(paste("Calculate", kind, "partial coex: START"), logLevel = 1L)
+    logThis(paste("Calculate", kind, "partial COEX: START"), logLevel = 1L)
 
     logThis(paste("Retrieving expected", kind, "partial contingency table"),
             logLevel = 3L)
@@ -1287,7 +1430,7 @@ calculatePartialCoex <- function(objCOTAN,
 
     gc()
 
-    logThis(paste("Calculating", kind, "partial coex normalization factor"),
+    logThis(paste("Calculating", kind, "partial COEX normalization factor"),
             logLevel = 3L)
 
     if (isTRUE(actOnCells)) {
@@ -1324,15 +1467,21 @@ calculatePartialCoex <- function(objCOTAN,
 
     gc()
 
-    # coex estimation
-    logThis(paste("Estimating", kind, "partial coex"), logLevel = 3L)
+    # COEX estimation
+    logThis(paste("Estimating", kind, "partial COEX"), logLevel = 3L)
 
     coex <- coex * (observedYY - expectedYY)
 
     rm(observedYY, expectedYY)
     gc()
 
-    logThis(paste("Calculate", kind, "partial coex: DONE"), logLevel = 1L)
+    endTime <- Sys.time()
+
+    logThis(paste("Total calculations elapsed time:",
+                  difftime(endTime, startTime, units = "secs")),
+            logLevel = 2L)
+
+    logThis(paste("Calculate", kind, "partial COEX: DONE"), logLevel = 1L)
 
     return(coex)
   }
@@ -1359,11 +1508,11 @@ calculatePartialCoex <- function(objCOTAN,
 #'
 calculateS <- function(objCOTAN, geneSubsetCol = vector(mode = "character"),
                        geneSubsetRow = vector(mode = "character")) {
+  logThis("Calculating S: START", logLevel = 3L)
+
   geneSubsetCol <- handleNamesSubsets(getGenes(objCOTAN), geneSubsetCol)
   geneSubsetRow <- handleNamesSubsets(getGenes(objCOTAN), geneSubsetRow)
 
-  logThis("Calculating S: START", logLevel = 3L)
-
   assert_that(isCoexAvailable(objCOTAN), msg = "Coex is missing")
 
   coex <- getGenesCoex(objCOTAN, zeroDiagonal = TRUE)
@@ -1403,11 +1552,13 @@ calculateS <- function(objCOTAN, geneSubsetCol = vector(mode = "character"),
 #'
 calculateG <- function(objCOTAN, geneSubsetCol = vector(mode = "character"),
                        geneSubsetRow = vector(mode = "character")) {
-  geneSubsetCol <- handleNamesSubsets(getGenes(objCOTAN), geneSubsetCol)
-  geneSubsetRow <- handleNamesSubsets(getGenes(objCOTAN), geneSubsetRow)
+  startTime <- Sys.time()
 
   logThis("Calculating G: START", logLevel = 2L)
 
+  geneSubsetCol <- handleNamesSubsets(getGenes(objCOTAN), geneSubsetCol)
+  geneSubsetRow <- handleNamesSubsets(getGenes(objCOTAN), geneSubsetRow)
+
   c(observedNN, observedNY, observedYN, observedYY) %<-%
     observedContingencyTables(objCOTAN, actOnCells = FALSE)
 
@@ -1451,7 +1602,248 @@ calculateG <- function(objCOTAN, geneSubsetCol = vector(mode = "character"),
   rm(tNN, tNY, tYN, tYY)
   gc()
 
+  endTime <- Sys.time()
+
+  logThis(paste("Total calculations elapsed time:",
+                difftime(endTime, startTime, units = "secs")),
+          logLevel = 2L)
+
   logThis("Calculating G: DONE", logLevel = 2L)
 
   return(G)
 }
+
+
+
+## -------- Genes selector -------
+
+#' @details `getSelectedGenes()` selects the *most representative* genes of the
+#'   `data.set`
+#'
+#' @param objCOTAN a `COTAN` object
+#' @param genesSel Decides whether and how to perform the gene-selection. used
+#'   for the clustering and the `UMAP`. It is a string indicating one of the
+#'   following selection methods:
+#'   * `"HGDI"` Will pick-up the genes with highest `GDI` (default)
+#'   * `"HVG_Seurat"` Will pick-up the genes with the highest variability
+#'   via the \pkg{Seurat} package
+#'   * `"HVG_Scanpy"` Will pick-up the genes with the highest variability
+#'   according to the `Scanpy` package (using the \pkg{Seurat} implementation)
+#' @param numGenes the number of genes to select using the above method. Will be
+#'   ignored when an explicit list of genes has been passed in
+#'
+#' @returns `getSelectedGenes()` returns an array with the genes' names
+#'
+#' @export
+#'
+#' @importFrom assertthat assert_that
+#'
+#' @importFrom Seurat CreateSeuratObject
+#' @importFrom Seurat NormalizeData
+#' @importFrom Seurat FindVariableFeatures
+#' @importFrom Seurat VariableFeatures
+#'
+#' @rdname CalculatingCOEX
+
+getSelectedGenes <- function(objCOTAN, genesSel = "", numGenes = 2000L) {
+  logThis("Running genes' selection: START", logLevel = 2L)
+
+  numGenes <- min(numGenes, getNumGenes(objCOTAN))
+  selectedGenes <- NULL
+
+  getHighestGDIGenes <- function(objCOTAN, numGenes) {
+    gdi <- suppressWarnings(getGDI(objCOTAN))
+    if (is_empty(gdi)) {
+      gdi <- getColumnFromDF(calculateGDI(objCOTAN, statType = "S",
+                                          rowsFraction = 0.05), "GDI")
+    }
+
+    sortedCandidates <- names(gdi)[order(gdi, decreasing = TRUE)]
+    if (gdi[sortedCandidates[numGenes]] < 1.3) {
+      numLowGDIGenes <- sum(gdi[sortedCandidates[seq_len(numGenes)]] <1.3)
+      logThis(paste("Included", numLowGDIGenes, "genes with GDI below 1.3"),
+              logLevel = 1L)
+    }
+    return(sortedCandidates[seq_len(numGenes)])
+  }
+
+  getHighestVariableGenes <- function(objCOTAN, numGenes, useVST) {
+    srat <- CreateSeuratObject(counts = getRawData(objCOTAN),
+                               project = "genes_selections")
+    srat <- NormalizeData(srat)
+
+    srat <- FindVariableFeatures(
+      srat, nfeatures = numGenes,
+      selection.method = ifelse(useVST, "vst", "mean.var.plot"))
+
+    return(VariableFeatures(object = srat))
+  }
+
+  selectedGenes <- NULL
+  if (length(genesSel) > 1L) {
+    assert_that(all(genesSel %in% getGenes(objCOTAN)),
+                msg = "Passed genes are not a subset of the relevant ones")
+
+    selectedGenes <- genesSel
+
+    logThis(paste("Given", length(selectedGenes), "genes as input"),
+            logLevel = 2L)
+  } else {
+    if (isEmptyName(genesSel)) {
+      genesSel = "HGDI"
+    }
+
+    selectedGenes <- switch(
+      genesSel,
+      HGDI = getHighestGDIGenes(objCOTAN, numGenes),
+      HVG_Seurat = , vst =
+        getHighestVariableGenes(objCOTAN, numGenes, TRUE),
+      HVG_Scanpy = , mean.var.plot =
+        getHighestVariableGenes(objCOTAN, numGenes, FALSE),
+      stop("Unrecognised `genesSel` passed in: ", genesSel)
+    )
+
+    logThis(paste("Selected", length(selectedGenes), "genes using",
+                  genesSel, "selector"), logLevel = 3L)
+  }
+
+  # align to sequence in input data
+  selectedGenes <- getGenes(objCOTAN)[getGenes(objCOTAN) %in% selectedGenes]
+
+  logThis("Running genes' selection: DONE", logLevel = 2L)
+
+  return(selectedGenes)
+}
+
+
+#' @details `calculateReducedDataMatrix()` calculates the reduced data-matrix to
+#'   be used for *clusterizations* or `UMAP` plots.
+#'
+#'   It uses the given `dataMethod` to determine with which data to start, then,
+#'   depending on the value of `useCoexEigen`, either uses the `genesSel` to
+#'   restrict evaluation to the relevant genes' before the `PCA` is run, or it
+#'   calculates the first **COEX** eigenvectors and projects the data matrix to
+#'   their sub-space.
+#'
+#' @param objCOTAN a `COTAN` object
+#' @param useCoexEigen Boolean to determine whether to project the data `matrix`
+#'   onto the first eigenvectors of the **COEX** `matrix` or instead restrict
+#'   the data `matrix` to the selected genes before applying the `PCA` reduction
+#' @param dataMethod selects the method to use to create the `data.frame` to
+#'   pass to the [UMAPPlot()]. To calculate, for each cell, a statistic for each
+#'   gene based on available data/model, the following methods are supported:
+#'   * `"RW", "Raw", "RawData"` uses the *raw* counts
+#'   * `"NN", "NuNorm", "Normalized"` uses the \eqn{\nu}*-normalized* counts
+#'   * `"LN", "LogNorm", "LogNormalized"` uses the *log-normalized* counts
+#'   (default)
+#'   * `"BI", "Bin", "Binarized"` uses the *binarized* data matrix
+#'   * `"BD", "BinDiscr", "BinarizedDiscrepancy"` uses the *difference* between
+#'   the *binarized* data matrix and the estimated *probability of one*
+#'   * `"AB", "AdjBin", "AdjBinarized"` uses the absolute value of
+#'   the *binarized discrepancy* above
+#'   * `"LH", "Like", "Likelihood"` uses the *likelihood* of *binarized*
+#'   data matrix
+#'   * `"LL", "LogLike", "LogLikelihood"` uses the *log-likelihood*
+#'   of *binarized* data matrix
+#'   * `"DL", "DerLogL", "DerivativeLogLikelihood"` uses the *derivative* of
+#'   the *log-likelihood* of *binarized* data matrix
+#'   * `"SL", "SignLogL", "SignedLogLikelihood"` uses the *signed log-likelihood*
+#'   of *binarized* data matrix
+#'
+#'   For the last four options see [calculateLikelihoodOfObserved()] for more
+#'   details
+#' @param numComp Number of components of the reduced `matrix`, it defaults to
+#'   25L.
+#' @param genesSel Decides whether and how to perform the gene-selection. used
+#'   for the clustering and the `UMAP`. It is a string indicating one of the
+#'   following selection methods:
+#'   * `"HGDI"` Will pick-up the genes with highest `GDI` (default)
+#'   * `"HVG_Seurat"` Will pick-up the genes with the highest variability
+#'   via the \pkg{Seurat} package
+#'   * `"HVG_Scanpy"` Will pick-up the genes with the highest variability
+#'   according to the `Scanpy` package (using the \pkg{Seurat} implementation)
+#' @param numGenes the number of genes to select using the above method. Will be
+#'   ignored when an explicit list of genes has been passed in
+#'
+#' @returns `calculateReducedDataMatrix()` returns the reduced matrix. The
+#'   returned `matrix` has dimensions: (number of cells, number of components)
+#'
+#' @importFrom zeallot %<-%
+#' @importFrom zeallot %->%
+#'
+#' @importFrom RSpectra eigs_sym
+#'
+#' @importFrom BiocSingular runPCA
+#'
+#' @export
+#'
+#' @rdname CalculatingCOEX
+#'
+calculateReducedDataMatrix <-
+  function(objCOTAN, useCoexEigen = FALSE,
+           dataMethod = "", numComp = 25L,
+           genesSel = "", numGenes = 2000L) {
+  startTime <- Sys.time()
+
+  logThis("Elaborating Reduced dimensionality Data Matrix - START",
+          logLevel = 2L)
+
+  numComp <- min(numComp, getNumGenes(objCOTAN))
+
+  cellsRDM <- NULL
+  if (isTRUE(useCoexEigen)) {
+    logThis("Elaborating COEX Eigen Vectors - START", logLevel = 3L)
+
+    # calculate the most relenvat eigen-vectors of the COEX matrix
+    res <- eigs_sym(as.matrix(getGenesCoex(objCOTAN, zeroDiagonal = FALSE)),
+                    k = numComp, which = "LM")
+    eigenVectors <- res$vectors[, seq_len(numComp)]
+    rm(res)
+    gc()
+
+    logThis("Elaborating COEX Eigen Vectors - DONE", logLevel = 3L)
+
+    # retrieve the the data matrix and project it onto the coex eigen-space
+    dataMatrix <- t(eigenVectors) %*%
+                    getDataMatrix(objCOTAN, dataMethod = dataMethod)
+
+    # re-scale in the cells direction
+    cellsRDM <- t(scale(dataMatrix, center = FALSE, scale = TRUE))
+    rm(dataMatrix)
+
+    colnames(cellsRDM) <- paste0("EC_", seq_len(ncol(cellsRDM)))
+  } else {
+    selectedGenes <- getSelectedGenes(objCOTAN, genesSel = genesSel,
+                                      numGenes = numGenes)
+    cellsMatrix <-
+      t(getDataMatrix(objCOTAN, dataMethod = dataMethod)[selectedGenes, ])
+
+    # re-scale so that all the genes have mean 0.0 and stdev 1.0
+    cellsMatrix <- scale(cellsMatrix, center = TRUE, scale = TRUE)
+
+    logThis("Elaborating PCA - START", logLevel = 3L)
+
+    cellsRDM <- runPCA(x = cellsMatrix,
+                       rank = min(numComp, ncol(cellsMatrix)),
+                       BSPARAM = IrlbaParam(),
+                       get.rotation = FALSE)[["x"]]
+    rm(cellsMatrix)
+
+    logThis("Elaborating PCA - DONE", logLevel = 3L)
+  }
+  gc()
+
+  assert_that(nrow(cellsRDM) == getNumCells(objCOTAN),
+              ncol(cellsRDM) <= numComp)
+
+  endTime <- Sys.time()
+
+  logThis(paste("Total calculations elapsed time:",
+                difftime(endTime, startTime, units = "secs")),
+          logLevel = 2L)
+
+  logThis("Elaborating Reduced dimensionality Data Matrix - DONE",
+          logLevel = 2L)
+
+  return(cellsRDM)
+}
