@@ -2,15 +2,15 @@
 # --------------------- Uniform Clusters ----------------------
 
 #' @details `mergeUniformCellsClusters()` takes in a **uniform**
-#'   *clusterization* and iteratively checks whether merging two *near clusters*
-#'   would form a **uniform** *cluster* still. Multiple thresholds will be used
-#'   from \eqn{1.37} up to the given one in order to prioritize merge of the
-#'   best fitting pairs.
+#'   *clusterization* and progressively checks whether *merging* two *near
+#'   clusters* would form a **uniform** *cluster* still. Multiple thresholds
+#'   will be used from \eqn{1.37} up to the given one in order to prioritize
+#'   merge of the best fitting pairs.
 #'
 #'   This function uses the *cosine distance* to establish the *nearest clusters
 #'   pairs*. It will use the [checkClusterUniformity()] function to check
 #'   whether the merged *clusters* are **uniform**. The function will stop once
-#'   no *tested pairs* of clusters are mergeable after testing all pairs in a
+#'   no *tested pairs* of clusters can be *merged* after testing all pairs in a
 #'   single batch
 #'
 #' @param objCOTAN a `COTAN` object
@@ -94,7 +94,9 @@
 #'                                          cores = 6L,
 #'                                          saveObj = FALSE)
 #'
-#' groupMarkers <- list(G1 = c("g-000010", "g-000020", "g-000030"),
+#' objCOTAN <- storeGDI(objCOTAN, genesGDI = calculateGDI(objCOTAN, cores = 6L))
+#'
+#' groupMarkers <- list(G1 = c("g-000010", "g-000020", "g-000138"),
 #'                      G2 = c("g-000300", "g-000330"),
 #'                      G3 = c("g-000510", "g-000530", "g-000550",
 #'                             "g-000570", "g-000590"))
@@ -110,14 +112,17 @@
 #' ##
 #'
 #' checker <- new("AdvancedGDIUniformityCheck")
-#' identical(checker@firstCheck@GDIThreshold, 1.297)
+#' stopifnot(identical(checker@firstCheck@GDIThreshold, 1.297))
 #'
 #' checker2 <- shiftCheckerThresholds(checker, 0.1)
-#' identical(checker2@firstCheck@GDIThreshold, 1.397)
+#' stopifnot(identical(checker2@firstCheck@GDIThreshold, 1.397))
 #'
 #' splitList <- cellsUniformClustering(objCOTAN, cores = 6L,
-#'                                     optimizeForSpeed = TRUE,
-#'                                     deviceStr = "cuda",
+#'                                     dataMethod = "LogLikelihood",
+#'                                     useCoexEigen = TRUE,
+#'                                     genesSel = "HGDI",
+#'                                     numGenes = 2000L,
+#'                                     numReducedComp = 50L,
 #'                                     initialResolution = 0.8,
 #'                                     checker = checker2,
 #'                                     saveObj = FALSE)
@@ -128,7 +133,7 @@
 #'
 #' checkerRes <-
 #'   checkClusterUniformity(objCOTAN, checker = checker2,
-#'                          cluster = clusters[[1L]], cells = firstCluster,
+#'                          clusterName = clusters[[1L]], cells = firstCluster,
 #'                          cores = 6L, optimizeForSpeed = TRUE,
 #'                          deviceStr = "cuda", saveObj = FALSE)
 #'
@@ -138,7 +143,7 @@
 #'                               coexDF = splitList[["coex"]],
 #'                               override = FALSE)
 #'
-#' identical(reorderClusterization(objCOTAN)[["clusters"]], clusters)
+#' stopifnot(identical(reorderClusterization(objCOTAN)[["clusters"]], clusters))
 #'
 #' ## It is possible to pass a list of checkers tot the merge function that will
 #' ## be applied each to the *resulting* merged *clusterization* obtained using
@@ -147,7 +152,6 @@
 #'
 #' mergedList <- mergeUniformCellsClusters(objCOTAN,
 #'                                         checkers = c(checker, checker2),
-#'                                         batchSize = 2L,
 #'                                         clusters = clusters,
 #'                                         cores = 6L,
 #'                                         optimizeForSpeed = TRUE,
@@ -162,7 +166,8 @@
 #'                               coexDF = mergedList[["coex"]],
 #'                               override = TRUE)
 #'
-#' identical(reorderClusterization(objCOTAN), mergedList[["clusters"]])
+#' stopifnot(identical(reorderClusterization(objCOTAN)[["clusters"]],
+#'                     mergedList[["clusters"]]))
 #'
 #' @rdname UniformClusters
 #'
@@ -191,7 +196,7 @@ mergeUniformCellsClusters <- function(objCOTAN,
   fromMergedName <- function(mergedClName, currentClNames) {
     partialMatch <- vapply(currentClNames, function(clName, mergedName) {
       return(str_detect(mergedName, clName))
-    }, FUN.VAL = logical(1L), mergedClName)
+    }, FUN.VALUE = logical(1L), mergedClName)
 
     c(clName1, clName2) %<-% c("", "")
     if (sum(partialMatch) == 2L) {
@@ -219,8 +224,8 @@ mergeUniformCellsClusters <- function(objCOTAN,
 
     untestedPairs <-
       vapply(pNamesList, function(pName, check, allRes) {
-        # we exploit the convention that a non-avalable result is marked by a
-        # zero clusterSize
+        # we exploit the convention that
+        # a non-available result is marked by a zero clusterSize
         updateChecker(pName, check, allRes)@clusterSize == 0L
       }, FUN.VALUE = logical(1L), checker, allCheckResults)
 
@@ -371,16 +376,18 @@ mergeUniformCellsClusters <- function(objCOTAN,
 
   # start analysis
 
+  startTime <- Sys.time()
+
   logThis("Merging cells' uniform clustering: START", logLevel = 2L)
 
   assert_that(estimatorsAreReady(objCOTAN),
-              msg = paste("Estimators lambda, nu, dispersion are not ready:",
-                          "Use proceedToCoex() to prepare them"))
+              msg = paste("Estimators `lambda`, `nu`, `dispersion` are not",
+                          "ready: Use proceedToCoex() to prepare them"))
 
   if (is_empty(checkers)) {
     GDIThreshold <- ifelse(is.finite(GDIThreshold), GDIThreshold, 1.43)
 
-    # keep old behaviour
+    # keep old behavior
     thresholdGap <- max(GDIThreshold - 1.37, 0.0)
     numThresholds <- ceiling(thresholdGap / 0.03)
     allThresholds <- 1.37 +
@@ -450,6 +457,8 @@ mergeUniformCellsClusters <- function(objCOTAN,
                    getCheckerThreshold(checker)),
             logLevel = 2L)
     repeat {
+      startLoopTime <- Sys.time()
+
       iter <- iter + 1L
       logThis(paste0("Start merging nearest clusters: iteration ", iter),
               logLevel = 3L)
@@ -538,6 +547,12 @@ mergeUniformCellsClusters <- function(objCOTAN,
         logThis(paste("Executed", (oldNumClusters - newNumClusters), "merges"),
                 logLevel = 3L)
       }
+
+      endLoopTime <- Sys.time()
+
+      logThis(paste("Loop calculations elapsed time:",
+                    difftime(endLoopTime, startLoopTime, units = "secs")),
+              logLevel = 2L)
     }
     logThis(paste0("Executed all merges for threshold ",
                    getCheckerThreshold(checker), " out of ",
@@ -561,8 +576,7 @@ mergeUniformCellsClusters <- function(objCOTAN,
       clTagsMap[["-1"]] <- "-1"
     }
 
-    outputClusters <- clTagsMap[outputClusters]
-    outputClusters <- set_names(outputClusters, getCells(objCOTAN))
+    outputClusters <- set_names(clTagsMap[outputClusters], getCells(objCOTAN))
 
     checksTokeep <- names(allCheckResults) %in% clTags
     allCheckResults <- allCheckResults[checksTokeep]
@@ -579,7 +593,7 @@ mergeUniformCellsClusters <- function(objCOTAN,
   c(outputClusters, outputCoexDF, permMap) %<-% tryCatch(
     reorderClusterization(objCOTAN, clusters = outputClusters,
                           coexDF = outputCoexDF, reverse = FALSE,
-                          keepMinusOne = FALSE, useDEA = useDEA,
+                          keepMinusOne = TRUE, useDEA = useDEA,
                           distance = distance, hclustMethod = hclustMethod),
     error = function(err) {
       logThis(paste("Calling reorderClusterization", err), logLevel = 0L)
@@ -590,8 +604,18 @@ mergeUniformCellsClusters <- function(objCOTAN,
   if (isTRUE(saveObj)) tryCatch({
     outFile <- file.path(outDirCond, "merge_check_results.csv")
     write.csv(checkersToDF(allCheckResults), file = outFile, na = "NaN")
+
+    outFile <- file.path(outDirCond, "merge_clusterization.csv")
+    write.csv(as.data.frame(list("merge" = outputClusters)),
+              file = outFile, na = "NaN")
   })
 
+  endTime <- Sys.time()
+
+  logThis(paste("Total calculations elapsed time:",
+                difftime(endTime, startTime, units = "secs")),
+          logLevel = 2L)
+
   logThis("Merging cells' uniform clustering: DONE", logLevel = 2L)
 
   return(list("clusters" = outputClusters, "coex" = outputCoexDF))
